@@ -11,7 +11,7 @@
 #' @docType package
 #' @name SeqMiner
 #' @useDynLib seqminer
-#' @importFrom utils
+#' @importFrom utils packageVersion
 NULL
 
 
@@ -287,8 +287,8 @@ rvmeta.readSkewByRange <- function(skewFile, tabixRange) {
 #' @seealso http://zhanxw.com/seqminer/ for online manual and examples
 #' @examples
 #' fileName = system.file("vcf/all.anno.filtered.extract.vcf.gz", package = "seqminer")
-#' snp <- tabix(fileName, "1:196623337-196632470")
-tabix <- function(tabixFile, tabixRange) {
+#' snp <- tabix.read(fileName, "1:196623337-196632470")
+tabix.read <- function(tabixFile, tabixRange) {
   stopifnot(file.exists(tabixFile))
   storage.mode(tabixFile) <- "character"
   storage.mode(tabixRange) <- "character"
@@ -443,3 +443,95 @@ rvmeta.readNullModel <- function(scoreTestFiles) {
   ret
 };
 
+#' Write score-based association statistics files.
+#'
+#' @param rvmetaData a list vector. It's usually read by rvmeta.readDataByRange or rvmeta.readDataByGene function
+#' @param outName character, a text indicating output file prefix
+#' @return TRUE only if succeed
+#' @export
+#' @seealso http://zhanxw.com/seqminer/ for online manual and examples
+#' @examples
+#' scoreFileName = system.file("rvtests/rvtest.MetaScore.assoc.anno.gz", package = "seqminer")
+#' covFileName = system.file("rvtests/rvtest.MetaCov.assoc.gz", package = "seqminer")
+#' geneFile = system.file("vcf/refFlat_hg19_6col.txt.gz", package = "seqminer")
+#' cfh <- rvmeta.readDataByRange(scoreFileName, covFileName, "1:196621007-196716634")
+#' rvmeta.writeScoreData(cfh, "cfh.MetaScore.assoc")
+rvmeta.writeScoreData <- function (rvmetaData, outName) {
+  storage.mode(outName) <- "character"
+  .Call("rvMetaWriteScoreData", rvmetaData, outName, PACKAGE="seqminer");
+  tabix.createIndex.meta(outName)
+}
+
+#' Write covariance association statistics files.
+#'
+#' @param rvmetaData a list vector. It's usually read by rvmeta.readDataByRange or rvmeta.readDataByGene function
+#' @param outName character, a text indicating output file prefix
+#' @return TRUE only if succeed
+#' @export
+#' @seealso http://zhanxw.com/seqminer/ for online manual and examples
+#' @examples
+#' scoreFileName = system.file("rvtests/rvtest.MetaScore.assoc.anno.gz", package = "seqminer")
+#' covFileName = system.file("rvtests/rvtest.MetaCov.assoc.gz", package = "seqminer")
+#' geneFile = system.file("vcf/refFlat_hg19_6col.txt.gz", package = "seqminer")
+#' cfh <- rvmeta.readDataByRange(scoreFileName, covFileName, "1:196621007-196716634")
+#' rvmeta.writeCovData(cfh, "cfh.MetaCov.assoc")
+rvmeta.writeCovData <- function (rvmetaData, outName) {
+  storage.mode(outName) <- "character"
+  .Call("rvMetaWriteCovData", rvmetaData, outName, PACKAGE="seqminer");
+  tabix.createIndex.meta(outName)  
+}
+
+#' Create tabix index file, similar to running tabix in command line.
+#'
+#' @param bgzipFile character, an tabix indexed file
+#' @param sequenceColumn  integer, sequence name column
+#' @param startColumn  integer, start column
+#' @param endColumn  integer, end column
+#' @param metaChar  character, symbol for comment/meta lines
+#' @param skipLines  integer, first this number of lines will be skipped
+#' @return NULL
+#' @export
+#' @seealso http://zhanxw.com/seqminer/ for online manual and examples
+#' @examples
+#' fileName = system.file("vcf/all.anno.filtered.extract.vcf.gz", package = "seqminer")
+#' tabix.createIndex(fileName, 1, 2, 0, '#', 0)
+tabix.createIndex <- function(bgzipFile, sequenceColumn = 1, startColumn = 4, endColumn = 5, metaChar = "#", skipLines = 0) {
+  stopifnot(file.exists(bgzipFile))
+  storage.mode(bgzipFile) <- "character"
+  storage.mode(sequenceColumn) <- "integer"
+  storage.mode(startColumn) <- "integer"
+  storage.mode(endColumn) <- "integer"
+  storage.mode(metaChar) <- "character"
+  storage.mode(skipLines) <- "integer"
+  .Call("createTabixIndex", bgzipFile, sequenceColumn, startColumn, endColumn, metaChar, skipLines, PACKAGE="seqminer");
+}
+
+#' Create tabix index for bgzipped VCF file
+#'
+#' @param bgzipVcfFile character, input vcf file
+#' @return NULL
+#' @export
+#' @seealso http://zhanxw.com/seqminer/ for online manual and examples
+#' @examples
+#' fileName = system.file("vcf/all.anno.filtered.extract.vcf.gz", package = "seqminer")
+#' tabix.createIndex.vcf(fileName)
+tabix.createIndex.vcf <- function(bgzipVcfFile) {
+  stopifnot(file.exists(bgzipVcfFile))
+  tabix.createIndex(bgzipVcfFile, 1, 2, 0, '#', 0)
+}
+
+#' Create tabix index for bgzipped MetaScore/MetaCov file
+#  MetaScore/MetaCov files can be generated by rvtests.
+#'
+#' @param bgzipFile character, input vcf file
+#' @return NULL
+#' @export
+#' @seealso http://zhanxw.com/seqminer/ for online manual and examples
+#' @seealso http://zhanxw.github.io/rvtests/ for rvtests
+#' @examples
+#' fileName = system.file("rvtests/rvtest.MetaScore.assoc.anno.gz", package = "seqminer")
+#' tabix.createIndex.meta(fileName)
+tabix.createIndex.meta <- function(bgzipFile) {
+  stopifnot(file.exists(bgzipFile))
+  tabix.createIndex(bgzipFile, 1, 2, 2, '#', 0)
+}
